"use strict";
this.name	= "snipergun";
this.author	= "Norby";
this.copyright	= "2016 Norby";
this.description= "Sniper Gun in subentity";
this.licence	= "CC BY-NC-SA 4.0";

//Sniper Guns are usable only for ships over certain mass and with a big gun subentity.

//equipment condition
this.allowAwardEquipment = function(equipment, ship, context)
{
        // OXP hook to allow stations to forbid specific equipment
        if (context == "purchase" && player.ship.dockedStation && player.ship.dockedStation.scriptInfo["oolite-barred-equipment"])
        {
                if (player.ship.dockedStation.scriptInfo["oolite-barred-equipment"].indexOf(equipment) != -1)
                {
                        return false;
                }
        }

        // OXP hook to allow ships to forbid specific "available to all" equipment
        if (ship.scriptInfo && ship.scriptInfo["oolite-barred-equipment"] && ship.scriptInfo["oolite-barred-equipment"].indexOf(equipment) != -1)
        {
                return false;
        }

//      player.consoleMessage( eqKey+" "+ship+" "+context );//debug

        return this._allowAwardSniperGun(equipment, ship, context, this);
}

this._allowAwardSniperGun = function(equipment, ship, context, w) //called from separatedlasers.js also
{

        //Sniper Gun can not fit into very small ships below 30t mass
        if( ship.mass < 30000 ) return false;

        var datakey = "snipergun"; //begin of dataKeys in shipdata.plist

        //Heavy Sniper Gun need 250t mass like Python
        if( equipment.indexOf("EQ_WEAPON_CANNON_HEAVY_SNIPER_GUN") > -1 ) {  //need indexOf instead of == for separated lasers
                if( ship.mass < 250000 ) return false;
                datakey = "snipergun-heavy"; //dataKey of Heavy Sniper Gun in shipdata.plist
        }


        //check if there is a proper subentity or snipergun_dummy on the ship
        var sub = w._checkSubEnt(ship, datakey, w);
//      log(w.name, ship.name+" "+equipment+" "+context+" sub="+sub); //debug

        if( sub >= 0 ) return true; //found
        else if( context == "scripted") {
            log(w.name, "Missing snipergun subentity on "+(ship.name)+" for "+
                equipment+" but awarded due to context is scripted");
            return true;
        }

        return false; //refused
}


//worldScript events
this.playerBoughtEquipment = function(equipmentKey) {
    this._validate(equipmentKey);
}

this.startUp = this.playerBoughtNewShip = function() {
        //search for sniper guns and validate
        this._validate("EQ_WEAPON_CANNON_SNIPER_GUN");
        this._validate("EQ_WEAPON_CANNON_HEAVY_SNIPER_GUN");
}


//snipergun functions

this._aft = function(o) { //check if o is oriented aft
        var v = Vector3D(0, 0, -1); //aft direction
        if( o.vectorForward().direction().distanceTo(v) == 0 ) return true; //exactly aft
        return false; //not exactly aft
}

this._checkAftSubEnt = function(ship, datakey, w) {
    return w._checkSubEnt2(ship, datakey, w, 2);
}

this._checkForwardSubEnt = function(ship, datakey, w) {
    return w._checkSubEnt2(ship, datakey, w, 1);
}

this._checkSubEnt = function(ship, datakey, w) {
    return w._checkSubEnt2(ship, datakey, w, 0);
}

this._checkSubEnt2 = function(ship, datakey, w, flag) {
        //check if there is a proper subentity or snipergun-dummy on the ship
        var sub = -1;
        var s = ship.subEntities;
        for(var i=0; i < s.length; i++) {
                if( s[i].dataKey.indexOf(datakey) == 0 ) { //matching at the first character
                    if( flag == 2 && w._aft(s[i].orientation) ) sub = i; //aft found
                    if( flag == 1 && !w._aft(s[i].orientation) ) sub = i; //forward found
                    if( flag == 0 ) sub = i; //either fw or aft found for allowAwardEquipment
//                  if(sub == i) log(w.name, s[i].dataKey+" subentity with orientation "+s[i].orientation+" flag:"+flag+(w._aft(s[i].orientation))); //debug
                }
        }
        return sub; //return the found index in subentity array or -1 if not found
}

this._refund = function(eq, length) { //refund weapon price, length is the size of multiple weapons array
        var f = 1;
        if( player.ship && player.ship.dockedStation ) {
                var e = player.ship.dockedStation.equipmentPriceFactor;
                if( e > 0 ) f = e;
        }
        if( 0 < oolite.compareVersion("1.83") ) length = 1; //no multiple weapons before Oolite 1.83
        var refund = f * length * EquipmentInfo.infoForKey( eq ).price/10;
        player.credits += refund;
        var m = EquipmentInfo.infoForKey( eq ).name+" removed from invalid mount, "+refund+" credits refunded.";
        player.consoleMessage(m, 10);
        log(this.name, m);
        return( refund );
}

this._validate = function(equipmentKey) {
    //called from playerBoughtEquipment, playerBoughtNewShip and startUp
    var datakey = false; //dataKey of snipergun subentity
    var sg = "EQ_WEAPON_CANNON_SNIPER_GUN";
    var hsg = "EQ_WEAPON_CANNON_HEAVY_SNIPER_GUN";
    if( equipmentKey.indexOf(sg) > -1 ) datakey = "snipergun"; //indexOf can check separated guns also
    if( equipmentKey.indexOf(hsg) > -1 ) datakey = "snipergun-heavy";
    if( datakey ) { //player bought a sniper gun
        var p = player.ship;
//      log(this.name, equipmentKey+" "+datakey+" "+p.forwardWeapon+" "+p.aftWeapon+" "+p.portWeapon+" "+p.starboardWeapon); //debug

        //check if there is a sniper gun in the ship
        //can not put sniper guns into side mounts
        if( p.portWeapon && p.portWeapon.equipmentKey == equipmentKey ) {
            this._refund( equipmentKey, p.weaponPositionPort.length );
            p.portWeapon = null;
        }

        if( p.starboardWeapon && p.starboardWeapon.equipmentKey == equipmentKey ) {
            this._refund( equipmentKey, p.weaponPositionStarboard.length  );
            p.starboardWeapon = null;
        }

        //check if there is a needed subentity in the ship
        //aft mount need aft orientation
        if( p.aftWeapon && p.aftWeapon.equipmentKey == equipmentKey ) {
            if( !(this._checkAftSubEnt(p, datakey, this) >= 0) ) {
                this._refund(equipmentKey, p.weaponPositionAft.length );
                p.aftWeapon = null;
            }
        }

        //forward mount need subentity with non-aft orientation
        if( p.forwardWeapon && p.forwardWeapon.equipmentKey == equipmentKey ) {
            if( !(this._checkForwardSubEnt(p, datakey, this) >= 0) ) {
                this._refund(equipmentKey, p.weaponPositionForward.length );
                p.forwardWeapon = null;
            }
        }

    }
}
